import subprocess

def get_broadcast_addresses():
    """
    Windows does not support broadcast to 255.255.255.255 so we need
    to enumerate the networks and generate separate network
    broadcast addresses instead.
    """
    # WSAIoctl is hard to access in python, so we do this the dirty way
    s = subprocess.Popen('ipconfig', stdout=subprocess.PIPE)
    # XXX: IPv4 only for now
    out = []
    addr = None
    for ln in s.stdout:
        ln = ln.strip()
        if not ln:
            addr = None
            continue
        if 'Address' in ln:
            addr = ln.split(':', 1)[-1].strip()
            continue
        if 'Mask' in ln:
            mask = ln.split(':', 1)[-1].strip()
            out.append('.'.join(
                str(int(a) | (int(m) ^ 255)) for a, m in zip(
                    addr.split('.'), mask.split('.'))))
    return out



import math
import random
from collections import namedtuple
import time
from pyrf.numpy_util import compute_fft
from pyrf.config import SweepEntry

class SweepStep(namedtuple('SweepStep', '''
        fcenter
        fstep
        fshift
        decimation
        points
        bins_skip
        bins_run
        bins_keep
        ''')):
    """
    Data structure used by SweepDevice for planning sweeps

    :param fcenter: starting center frequency in Hz
    :param fstep: frequency increment each step in Hz
    :param fshift: frequency shift in Hz
    :param decimation: decimation value
    :param points: samples to capture
    :param bins_skip: number of FFT bins to skip from left
    :param bins_run: number of usable FFT bins each step
    :param bins_keep: total number of bins to keep from all steps
    """
    __slots__ = []

    def to_sweep_entry(self, device, **kwargs):
        """
        Create a SweepEntry for device matching this SweepStep,

        extra parameters (gain, antenna etc.) may be provided as keyword
        parameters
        """
        if self.points > 32*1024:
            raise SweepDeviceError('large captures not yet supported')

        return SweepEntry(
            fstart=self.fcenter,
            fstop=min(self.fcenter + (self.steps + 0.5) * self.fstep,
                device.properties.MAX_TUNABLE),
            fstep=self.fstep,
            fshift=self.fshift,
            decimation=self.decimation,
            spp=self.points,
            ppb=1,
            **kwargs)

    @property
    def steps(self):
        return math.ceil(float(self.bins_keep) / self.bins_run)


    def trim(self, device, fstart, fstop):
        """
        Create a SweepStep that only includes steps within fstart to fstop
        or return None if no part of this SweepStep overlaps
        """
        if fstop <= fstart:
            return

        steps = self.steps
        bin_width = float(device.properties.FULL_BW
            ) / self.decimation / self.points
        start_centered = self.bins_skip - self.points / 2
        start_off = start_centered * bin_width - self.fshift
        stop_centered = start_centered + self.bins_run
        stop_off = stop_centered * bin_width - self.fshift

        start_step = math.floor((fstart - self.fcenter - start_off) / self.fstep)
        stop_step = steps + math.ceil(
            (fstop - self.fcenter - stop_off) / self.fstep)
        if steps <= start_step or stop_step <= 0:
            return

        last_bins_run = self.bins_keep % self.bins_run
        if steps - 1 == start_step and last_bins_run:
            # starting from last step special case: fstart might be
            # after the remaining samples
            last_stop_centered = start_centered + last_bins_run
            last_stop_off = last_stop_centered * bin_width - self.fshift
            last_stop = self.fcenter + steps * self.fstep + last_stop_off
            if last_stop <= fstart:
                return

        trim_left = max(0, start_step)
        trim_right = steps - min(steps, stop_step)
        return SweepStep(
            fcenter=self.fcenter + trim_left * self.fstep,
            fstep=self.fstep,
            fshift=self.fshift,
            decimation=self.decimation,
            points=self.points,
            bins_skip=self.bins_skip,
            bins_run=self.bins_run,
            bins_keep=self.bins_keep - (trim_left + trim_right) * self.bins_run,
            )


class SweepDeviceError(Exception):
    pass

class SweepDevice(object):
    """
    Virtual device that generates power levels from a range of
    frequencies by sweeping the frequencies with a real device
    and piecing together FFT results.

    :param real_device: device that will will be used for capturing data,
                        typically a :class:`pyrf.devices.thinkrf.WSA` instance.
    :param callback: callback to use for async operation (not used if
                     real_device is using a :class:`PlainSocketConnector`)
    """
    def __init__(self, real_device, async_callback=None):
        self.real_device = real_device
        self._sweep_id = random.randrange(0, 2**32-1) # don't want 2**32-1
        if hasattr(self.connector, 'vrt_callback'):
            if not async_callback:
                raise SweepDeviceError(
                    "async_callback required for async operation")
            # disable receiving data until we are expecting it
            self.connector.vrt_callback = None
        else:
            if async_callback:
                raise SweepDeviceError(
                    "async_callback not applicable for sync operation")
        self._prev_sweep_id = None
        self._trigger_sweep = False
        self._trigger_id = None
        self._trigger_data = {}
        self.async_callback = async_callback
        self.continuous = False
        self.context_bytes_received = 0
        self.data_bytes_received = 0
        self.data_bytes_processed = 0
        self.martian_bytes_discarded = 0
        self.past_end_bytes_discarded = 0
        self.fft_calculation_seconds = 0.0
        self.bin_collection_seconds = 0.0

    connector = property(lambda self: self.real_device.connector)

    def capture_power_spectrum(self,
            fstart, fstop, rbw, device_settings=None,
            triggers=None, continuous=False,
            min_points=128, max_points=8192):
        """
        Initiate a capture of power spectral density by
        setting up a sweep list and starting a single sweep.

        :param fstart: starting frequency in Hz
        :type fstart: float
        :param fstop: ending frequency in Hz
        :type fstop: float
        :param rbw: requested RBW in Hz (output RBW may be smaller than
                    requested)
        :type rbw: float
        :param device_settings: antenna, gain and other device settings
        :type dict:
        :param triggers: list of :class:`TriggerSettings` instances or None
        :param continuous: async continue after first sweep
        :type continuous: bool
        :param min_points: smallest number of points per capture from real_device
        :type min_points: int
        :param max_points: largest number of points per capture from real_device
                           (due to decimation limits points returned may be larger)
        :type max_points: int

        When triggers are provided nothing will be captured until one of the
        triggers is satisfied. The trigger data received is combined with a full
        sweep before being returned.
        """
        if continuous and not self.async_callback:
            raise SweepDeviceError(
                "continuous mode only applies to async operation")
        self.device_settings = device_settings
        self.continuous = continuous

        self.real_device.abort()
        self.real_device.flush()
        self.real_device.request_read_perm()

        self.fstart, self.fstop, self.plan = plan_sweep(self.real_device,
            fstart, fstop, rbw, min_points, max_points)

        result = self._perform_trigger_sweep(triggers)
        if result == 'async waiting':
            return

        return self._perform_full_sweep()


    def _perform_trigger_sweep(self, triggers):
        entries = []
        if not triggers:
            return
        for t in triggers:
            if t.trigtype != 'LEVEL':
                raise SweepDeviceError('only level triggers supported')
            tplan = trim_sweep_plan(self.real_device,
                self.plan, t.fstart, t.fstop)
            for ss in tplan:
                entries.append(ss.to_sweep_entry(self.real_device,
                    dwell_us=1,
                    trigtype=t.trigtype,
                    level_fstart=t.fstart,
                    level_fstop=t.fstop,
                    level_amplitude=t.amplitude,
                    **self.device_settings))
        if not entries:
            return

        if self.async_callback:
            self.connector.vrt_callback = self._vrt_receive
            self._start_sweep(entries, trigger=True)
            return 'async waiting'
        assert not self._trigger_data
        self._start_sweep(entries, trigger=True)
        while not self._trigger_data:
            result = self._vrt_receive(self.real_device.read())


    def _perform_full_sweep(self):
        entries = []
        for ss in self.plan:
            entries.append(ss.to_sweep_entry(self.real_device,
                **self.device_settings))

        if self.async_callback:
            if not self.plan:
                self.async_callback(self.fstart, self.fstop, [])
                return
            self.connector.vrt_callback = self._vrt_receive
            self._start_sweep(entries)
            return

        if not self.plan:
            return (self.fstart, self.fstop, [])
        self._start_sweep(entries)
        result = None
        while result is None:
            result = self._vrt_receive(self.real_device.read())
        return result

    def _start_sweep(self, entries, trigger=False):
        self.real_device.abort()
        self.real_device.flush()
        self.real_device.sweep_clear()
        assert entries, "starting sweep with no sweep entries"
        for e in entries:
            self.real_device.sweep_add(e)
        self._prev_sweep_id = self._sweep_id
        self._sweep_id = (self._sweep_id + 1) & (2**32 - 1)
        self._vrt_context = {}
        self._ss_index = 0
        self._ss_received = 0
        self.bins = []
        self.real_device.sweep_iterations(
            0 if trigger or self.continuous else 1)
        self.real_device.sweep_start(self._sweep_id)
        self._trigger_sweep = trigger
        if trigger:
            self._trigger_id = self._sweep_id

    def _vrt_receive(self, packet):
        packet_bytes = packet.size * 4

        if packet.is_context_packet():
            self._vrt_context.update(packet.fields)
            self.context_bytes_received += packet_bytes
            return

        self.data_bytes_received += packet_bytes
        sweep_id = self._vrt_context.get('sweepid')
        if sweep_id not in (self._sweep_id, self._trigger_id):
            if sweep_id == self._prev_sweep_id:
                self.past_end_bytes_discarded += packet_bytes
            else:
                self.martian_bytes_discarded += packet_bytes
            return # not our data
        # WORKAROUND for 5K not always sending reflevel
        if 'reflevel' not in self._vrt_context:
            self._vrt_context['reflevel'] = 0
        assert 'reflevel' in self._vrt_context, (
            "missing required context, sweep failed")

        freq = self._vrt_context['rffreq']
        if self._trigger_sweep or sweep_id == self._trigger_id:
            self._trigger_data[freq] = packet
            if self._trigger_sweep and self.async_callback:
                self._perform_full_sweep()
            return

        if self._ss_index is None:
            self.past_end_bytes_discarded += packet_bytes
            return # more data than we asked for

        fft_start_time = time.time()
        # replace with trigger data when available
        packet = self._trigger_data.get(freq, packet)
        pow_data = compute_fft(self.real_device, packet, self._vrt_context)
        # collect and compute bins
        collect_start_time = time.time()
        ss = self.plan[self._ss_index]
        take = min(ss.bins_run, ss.bins_keep - self._ss_received)
        self.bins.extend(pow_data[ss.bins_skip:ss.bins_skip + take])
        self._ss_received += take
        collect_stop_time = time.time()

        self.fft_calculation_seconds += collect_start_time - fft_start_time
        self.bin_collection_seconds += collect_stop_time - collect_start_time
        self.data_bytes_processed += take * 4

        if self._ss_received < ss.bins_keep:
            return

        self._ss_received = 0
        self._ss_index += 1
        if self._ss_index < len(self.plan):
            return

        # done the complete sweep
        # XXX: in case sweep_iterations() does not work
        if not self.continuous:
            self._ss_index = None
            self._trigger_data = {}
            self.real_device.abort()
            self.real_device.flush()

        if self.async_callback:
            self.real_device.vrt_callback = None
            self.async_callback(self.fstart, self.fstop, self.bins)
            if self.continuous:
                self._ss_index = 0
                self._ss_received = 0
                self.bins = []
            return
        return (self.fstart, self.fstop, self.bins)



def plan_sweep(device, fstart, fstop, rbw, min_points=128, max_points=8192):
    """
    :param device: a device class or instance such as
                   :class:`pyrf.devices.thinkrf.WSA`
    :param fstart: starting frequency in Hz
    :type fstart: float
    :param fstop: ending frequency in Hz
    :type fstop: float
    :param rbw: requested RBW in Hz (output RBW may be smaller than requested)
    :type rbw: float
    :param min_points: smallest number of points per capture
    :type min_points: int
    :param max_points: largest number of points per capture (due to
                       decimation limits points returned may be larger)
    :type max_points: int

    The following device properties are used in planning the sweep:

    device.properties.FULL_BW
      full width of the filter in Hz
    device.properties.USABLE_BW
      usable portion before filter drop-off at edges in Hz
    device.properties.MIN_TUNABLE
      the lowest valid center frequency for arbitrary tuning in Hz,
      0(DC) is always assumed to be available for direct digitization
    device.properties.MAX_TUNABLE
      the highest valid center frequency for arbitrart tuning in Hz
    device.properties.MIN_DECIMATION
      the lowest valid decimation value above 1, 1(no decimation) is
      assumed to always be available
    device.properties.MAX_DECIMATION
      the highest valid decimation value, only powers of 2 will be used
    device.properties.DECIMATED_USABLE
      the fraction decimated output containing usable data, float < 1.0
    device.properties.DC_OFFSET_BW
      the range of frequencies around center that may be affected by
      a DC offset and should not be used

    :returns: (actual fstart, actual fstop, list of SweepStep instances)

    The caller would then use each of these tuples to do the following:

    1. The first 5 values are used for a single capture or single sweep
    2. An FFT is run on the points returned to produce bins in the linear
       domain
    3. bins[bins_skip:bins_skip + bins_run] are selected
    4. take logarithm of output bins and appended to the result
    5. for sweeps repeat from 2 until the sweep is complete
    6. bins_keep is the total number of selected bins to keep; for
       single captures bins_run == bins_keep
    """
    prop = device.properties
    out = []
    usable2 = prop.USABLE_BW / 2.0
    dc_offset2 = prop.DC_OFFSET_BW / 2.0

    # FIXME: truncate to left-hand sweep area for now
    fstart = max(prop.MIN_TUNABLE - usable2, fstart)
    fstop = min(prop.MAX_TUNABLE - dc_offset2, fstop)

    if fstop <= fstart:
        return (fstart, fstart, [])

    points = prop.FULL_BW / rbw
    points = max(min_points, 2 ** math.ceil(math.log(points, 2)))

    decimation = 1
    ideal_decimation = 2 ** math.ceil(math.log(float(points) / max_points, 2))
    min_decimation = max(2, prop.MIN_DECIMATION)
    max_decimation = 2 ** math.floor(math.log(prop.MAX_DECIMATION, 2))
    if max_points < points and min_decimation <= ideal_decimation:
        decimation = min(max_decimation, ideal_decimation)
        points /= decimation
        decimated_bw = prop.FULL_BW / decimation
        decimation_edge_bins = math.ceil(points * prop.DECIMATED_USABLE / 2.0)
        decimation_edge = decimation_edge_bins * decimated_bw / points

    bin_size = prop.FULL_BW / decimation / float(points)

    # there are three regions that need to be handled differently
    # region 0: direct digitization / "VLOW band"
    if fstart < prop.MIN_TUNABLE - usable2:
        raise NotImplemented # yet

    # region 1: left-hand sweep area
    if prop.MIN_TUNABLE - usable2 <= fstart:
        if decimation == 1:
            left_edge = prop.FULL_BW / 2.0 - usable2
            left_bin = math.ceil(left_edge / bin_size)
            fshift = left_bin * bin_size - left_edge
            usable_bins = (usable2 - dc_offset2 - fshift) // bin_size
        else:
            left_bin = decimation_edge_bins
            fshift = usable2 + decimation_edge - (decimated_bw / 2.0)
            usable_bins = min(points - (decimation_edge_bins * 2),
                (usable2 - dc_offset2) // bin_size)

        usable_bw = usable_bins * bin_size

        fcenter = fstart + usable2
        # FIXME: fstop not being updated here
        max_steps = math.floor((prop.MAX_TUNABLE - fstart) / usable_bw)
        bins_keep = min(round((fstop - fstart) / bin_size),
            max_steps * usable_bins)
        sweep_steps = math.ceil(bins_keep / usable_bins)
        out.append(SweepStep(
            fcenter=fcenter,
            fstep=usable_bw,
            fshift=fshift,
            decimation=decimation,
            points=int(points),
            bins_skip=int(left_bin),
            bins_run=int(min(usable_bins, bins_keep)),
            bins_keep=int(bins_keep),
            ))

    # region 2: right-hand edge
    if prop.MAX_TUNABLE - dc_offset2 < fstop:
        raise NotImplemented # yet

    return (fstart, fstop, out)


def trim_sweep_plan(device, plan, fstart, fstop):
    """
    :param device: a device class or instance such as
                   :class:`pyrf.devices.thinkrf.WSA`
    :param plan: list of :class:`SweepStep` instances
    :param fstart: starting frequency in Hz
    :type fstart: float
    :param fstop: ending frequency in Hz
    :type fstop: float

    produce a new sweep plan consisting of captures from the passed
    sweep plan that overlap with the range fstart to fstop.
    """

    trimmed = (ss.trim(device, fstart, fstop) for ss in plan)
    return [t for t in trimmed if t]





"""
The main application window and GUI controls

``MainWindow`` creates and handles the ``File | Open Device`` menu and
wraps the ``MainPanel`` widget responsible for most of the interface.

All the buttons and controls and their callback functions are built in
``MainPanel`` and arranged on a grid.  A ``Pyqtgraph Window`` is created
and placed to left of the controls.
"""

import sys
from PySide import QtGui, QtCore
import numpy as np
import time
from contextlib import contextmanager
from util import find_max_index, find_nearest_index
from util import hotkey_util
import constants
import control_util as cu
from plot_widget import plot
import gui_config
from pyrf.devices.thinkrf import WSA
from pyrf.sweep_device import SweepDevice
from pyrf.connectors.twisted_async import TwistedConnector
from pyrf.config import TriggerSettings
from pyrf.capture_device import CaptureDevice

try:
    from twisted.internet.defer import inlineCallbacks
except ImportError:
    def inlineCallbacks(fn):
        pass

WINDOW_WIDTH = 1400
WINDOW_HEIGHT = 400

class MainWindow(QtGui.QMainWindow):
    """
    The main window and menus
    """
    def __init__(self, name=None):
        super(MainWindow, self).__init__()
        self.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
        self.initUI()


        self.show()
    
    def initUI(self):
        name = None
        if len(sys.argv) > 1:
            name = sys.argv[1]
        self.mainPanel = MainPanel()
        openAction = QtGui.QAction('&Open Device', self)
        openAction.triggered.connect( self.mainPanel.open_device_dialog)
        exitAction = QtGui.QAction('&Exit', self)
        exitAction.setShortcut('Ctrl+Q')
        exitAction.triggered.connect(self.close)
        menubar = self.menuBar()
        fileMenu = menubar.addMenu('&File')
        fileMenu.addAction(openAction)
        fileMenu.addAction(exitAction)
        self.setWindowTitle('Spectrum Analyzer')
        self.setCentralWidget(self.mainPanel)
        self.mainPanel.show()
        if name:
            self.mainPanel.open_device(name)
        else:
            self.mainPanel.open_device_dialog()
    
    def closeEvent(self, event):
        if self.mainPanel.dut:
            self.mainPanel.dut.abort()
            self.mainPanel.dut.flush()
            self.mainPanel.dut.reset()
        event.accept()
        self.mainPanel._reactor.stop()


class MainPanel(QtGui.QWidget):
    """
    The spectrum view and controls
    """
    def __init__(self):
        self.dut = None
        self.control_widgets = []
        super(MainPanel, self).__init__()
        self.setMinimumWidth(800)
        self.setMinimumHeight(600)
        self.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
        self.plot_state = gui_config.plot_state()
        # plot window
        self._plot = plot(self)
        self._vrt_context = {}
        self.initUI()
        self.disable_controls()
        self._reactor = self._get_reactor()

    def _get_reactor(self):
        # late import because installReactor is being used
        from twisted.internet import reactor
        return reactor

    def open_device_dialog(self):
        name, ok = QtGui.QInputDialog.getText(self, 'Open Device',
            'Enter a hostname or IP address:')
        while True:
            if not ok:
                return
            try:
                self.open_device(name)
                break
            except socket.error:
                name, ok = QtGui.QInputDialog.getText(self, 'Open Device',
                    'Connection Failed, please try again\n\n'
                    'Enter a hostname or IP address:')

    @inlineCallbacks
    def open_device(self, name):
        dut = WSA(connector=TwistedConnector(self._reactor))
        yield dut.connect(name)
        self.dut = dut
        if dut.properties.model == 'WSA5000':
            self._antenna_box.hide()
            self._gain_box.hide()
            self._trigger.hide()
            #self._attenuator_box.show()
        else:
            self._antenna_box.show()
            self._gain_box.show()
            self._trigger.show()
            #self._attenuator_box.hide()

        self.sweep_dut = SweepDevice(dut, self.receive_data)
        self.cap_dut = CaptureDevice(dut, self.receive_data)
        self.enable_controls()
        cu._select_fstart(self)
        self.read_sweep()

    def read_sweep(self):
        #TODO: find cleaner way to do this
        self.plot_state.dev_set.pop('freq', None)
        self.plot_state.dev_set.pop('trigger', None)
        self.sweep_dut.capture_power_spectrum(self.plot_state.fstart,
                                              self.plot_state.fstop,
                                              self.plot_state.rbw,
                                              self.plot_state.dev_set,
                                              continuous = False)

    def read_trigg(self):
        device_set = self.plot_state.dev_set
        #TODO: find cleaner way to do this
        device_set['freq'] = self.plot_state.center_freq
        device_set['trigger'] = self.plot_state.trig_set

        self.cap_dut.capture_power_spectrum(device_set,self.plot_state.rbw)


    def receive_data(self, fstart, fstop, pow_):
        if not self.plot_state.enable_plot:
            return
        if self.plot_state.trig_set:
            self.read_trigg()
        else:
            self.read_sweep()
        self.pow_data = pow_
        self.update_plot()


    def keyPressEvent(self, event):
        if self.dut:
            hotkey_util(self, event)
           
    def mousePressEvent(self, event):
        if self.dut:
            if event.button() == QtCore.Qt.MouseButton.LeftButton:
                click_pos =  event.pos().x() - 68
                plot_window_width = self._plot.window.width() - 68

                if click_pos < plot_window_width and click_pos > 0:

                    window_freq = self._plot.view_box.viewRange()[0]
                    window_bw =  (window_freq[1] - window_freq[0])
                    click_freq = ((float(click_pos) / float(plot_window_width)) * float(window_bw)) + window_freq[0]

                    if self.plot_state.marker_sel:
                        self._marker.setDown(True)
                        self.plot_state.marker_ind  = find_nearest_index(click_freq, self.plot_state.freq_range)
                        self.update_marker()
                    
                    elif self.plot_state.delta_sel:
                        self._delta.setDown(True)
                        self.plot_state.delta_ind = find_nearest_index(click_freq, self.plot_state.freq_range)
                        self.update_delta()
                    self.update_diff()
    def initUI(self):
        grid = QtGui.QGridLayout()
        grid.setSpacing(10)
        for x in range(8):
            grid.setColumnMinimumWidth(x, 300)
        grid.setRowMinimumHeight(14, 800)

        # add plot widget
        plot_width = 8
        grid.addWidget(self._plot.window,0,0,15,plot_width)

        marker_label, delta_label, diff_label = self._marker_labels()
        grid.addWidget(marker_label, 0, 1, 1, 2)
        grid.addWidget(delta_label, 0, 3, 1, 2)
        grid.addWidget(diff_label , 0, 5, 1, 2)
 
        y = 0
        x = plot_width
        grid.addWidget(self._device_controls(), y, x, 2, 5)
        y += 2
        grid.addWidget(self._freq_controls(), y, x, 4, 5)
        y += 4
        grid.addWidget(self._plot_controls(), y, x, 4, 5)

        self.update_freq()
        self.setLayout(grid)

        
    
    def _device_controls(self):
        dev_group = QtGui.QGroupBox("Device Control")
        self.dev_group = dev_group
        
        dev_layout = QtGui.QVBoxLayout()
        
        first_row = QtGui.QHBoxLayout()
        first_row.addWidget(self._antenna_control())
        first_row.addWidget(self._trigger_control())
        first_row.addWidget(self._attenuator_control())
        
        second_row = QtGui.QHBoxLayout()
        second_row.addWidget(self._gain_control())
        second_row.addWidget(self._ifgain_control())
        
        dev_layout.addLayout(first_row)
        dev_layout.addLayout(second_row)

        dev_group.setLayout(dev_layout)         
        return dev_group
    def _antenna_control(self):
        antenna = QtGui.QComboBox(self)
        antenna.setToolTip("Choose Antenna") 
        antenna.addItem("Antenna 1")
        antenna.addItem("Antenna 2")
        self._antenna_box = antenna
        self.control_widgets.append(self._antenna_box)
        def new_antenna():
            self.plot_state.dev_set['antenna'] = (int(antenna.currentText().split()[-1]))
        
        antenna.currentIndexChanged.connect(new_antenna)
        return antenna

    def _gain_control(self):
        gain = QtGui.QComboBox(self)
        gain.setToolTip("Choose RF Gain setting") 
        gain_values = ['VLow', 'Low', 'Med', 'High']
        for g in gain_values:
            gain.addItem("RF Gain: %s" % g)
        self._gain_values = [g.lower() for g in gain_values]
        self._gain_box = gain
        self.control_widgets.append(self._gain_box)
        def new_gain():
            self.plot_state.dev_set['gain'] = gain.currentText().split()[-1].lower().encode('ascii')
        gain.currentIndexChanged.connect(new_gain)
        return gain

    def _ifgain_control(self):
        ifgain = QtGui.QSpinBox(self)
        ifgain.setToolTip("Choose IF Gain setting")
        ifgain.setRange(-10, 25)
        ifgain.setSuffix(" dB")
        self._ifgain_box = ifgain
        self.control_widgets.append(self._ifgain_box)
        def new_ifgain():
            self.plot_state.dev_set['ifgain'] = ifgain.value()
        ifgain.valueChanged.connect(new_ifgain)
        return ifgain
    
    def _trigger_control(self):
        trigger = QtGui.QCheckBox("Trigger")
        trigger.setToolTip("[T]\nTurn the Triggers on/off") 
        trigger.clicked.connect(lambda: cu._trigger_control(self))
        self._trigger = trigger
        self.control_widgets.append(self._trigger)
        return trigger

    def _attenuator_control(self):
        attenuator = QtGui.QCheckBox("Attenuator")
        attenuator.setChecked(True)
        def new_attenuator():
            self.plot_state.dev_set['attenuator'] = attenuator.isChecked()
        attenuator.clicked.connect(new_attenuator)
        self._attenuator_box = attenuator
        self.control_widgets.append(attenuator)
        return attenuator
    
    def _freq_controls(self):
        freq_group = QtGui.QGroupBox("Frequency Control")
        self._freq_group = freq_group
        
        freq_layout = QtGui.QVBoxLayout()
        
        fstart_hbox = QtGui.QHBoxLayout()
        fstart_bt, fstart_txt = self._fstart_controls()
        fstart_hbox.addWidget(fstart_bt)
        fstart_hbox.addWidget(fstart_txt)
        fstart_hbox.addWidget(QtGui.QLabel('MHz'))
        
        cfreq_hbox = QtGui.QHBoxLayout()
        cfreq_bt, cfreq_txt = self._center_freq()
        cfreq_hbox.addWidget(cfreq_bt)
        cfreq_hbox.addWidget(cfreq_txt)
        cfreq_hbox.addWidget(QtGui.QLabel('MHz'))
        
        bw_hbox = QtGui.QHBoxLayout()
        bw_bt, bw_txt = self._bw_controls()
        bw_hbox.addWidget(bw_bt)
        bw_hbox.addWidget(bw_txt)
        bw_hbox.addWidget(QtGui.QLabel('MHz'))
        
        fstop_hbox = QtGui.QHBoxLayout()
        fstop_bt, fstop_txt = self._fstop_controls()
        fstop_hbox.addWidget(fstop_bt)
        fstop_hbox.addWidget(fstop_txt)
        fstop_hbox.addWidget(QtGui.QLabel('MHz'))
        
        freq_inc_hbox = QtGui.QHBoxLayout()
        freq_inc_steps, freq_inc_minus, freq_inc_plus = self._freq_incr()
        freq_inc_hbox.addWidget(freq_inc_minus)
        freq_inc_hbox.addWidget(freq_inc_steps)
        freq_inc_hbox.addWidget(freq_inc_plus)
        
        rbw_hbox = QtGui.QHBoxLayout()
        rbw = self._rbw_controls()
        rbw_hbox.addWidget(QtGui.QLabel('Resolution Bandwidth:'))
        rbw_hbox.addWidget(rbw)
        
        freq_layout.addLayout(fstart_hbox)
        freq_layout.addLayout(cfreq_hbox)
        freq_layout.addLayout(bw_hbox)
        freq_layout.addLayout(fstop_hbox)
        freq_layout.addLayout(freq_inc_hbox)
        freq_layout.addLayout(rbw_hbox)
        freq_group.setLayout(freq_layout)
        
        return freq_group
    def _center_freq(self):
        cfreq = QtGui.QPushButton('Center')
        cfreq.setToolTip("[2]\nTune the center frequency") 
        self._cfreq = cfreq
        cfreq.clicked.connect(lambda: cu._select_center_freq(self))
        freq_edit = QtGui.QLineEdit(str(self.plot_state.center_freq/constants.MHZ))
        self._freq_edit = freq_edit
        self.control_widgets.append(self._cfreq)
        self.control_widgets.append(self._freq_edit)
        def freq_change():
            cu._select_center_freq(self)
            self.update_freq()
            self.update_freq_edit()
        
        freq_edit.returnPressed.connect(lambda: freq_change())
        return cfreq, freq_edit
    
    def _freq_incr(self):
        steps = QtGui.QComboBox(self)
        steps.addItem("Adjust: 1 MHz")
        steps.addItem("Adjust: 2.5 MHz")
        steps.addItem("Adjust: 10 MHz")
        steps.addItem("Adjust: 25 MHz")
        steps.addItem("Adjust: 100 MHz")
        self.fstep = float(steps.currentText().split()[1])
        def freq_step_change():
            self.fstep = float(steps.currentText().split()[1])
        steps.currentIndexChanged.connect(freq_step_change)
        steps.setCurrentIndex(2)
        self._fstep_box = steps
        def freq_step(factor):
            try:
                f = float(self._freq_edit.text())
            except ValueError:
                return
            delta = float(steps.currentText().split()[1]) * factor
            self.update_freq(delta)
            self.update_freq_edit()   
        freq_minus = QtGui.QPushButton('-')
        freq_minus.clicked.connect(lambda: freq_step(-1))
        self._freq_minus = freq_minus
        freq_plus = QtGui.QPushButton('+')
        freq_plus.clicked.connect(lambda: freq_step(1))
        self._freq_plus = freq_plus
        self.control_widgets.append(self._freq_plus)
        self.control_widgets.append(self._freq_minus)
        self.control_widgets.append(self._fstep_box)
        return  steps, freq_plus, freq_minus
    
    
    def _bw_controls(self):
        bw = QtGui.QPushButton('Span')
        bw.setToolTip("[3]\nChange the bandwidth of the current plot")
        self._bw = bw
        bw.clicked.connect(lambda: cu._select_bw(self))
        bw_edit = QtGui.QLineEdit(str(self.plot_state.bandwidth/constants.MHZ))
        def freq_change():
            cu._select_bw(self)
            self.update_freq()
            self.update_freq_edit()   
        bw_edit.returnPressed.connect(lambda: freq_change())
        self._bw_edit = bw_edit
        self.control_widgets.append(self._bw_edit)
        self.control_widgets.append(self._bw)
        return bw, bw_edit
    
    def _fstart_controls(self):
        fstart = QtGui.QPushButton('Start')
        fstart.setToolTip("[1]\nTune the start frequency")
        self._fstart = fstart
        fstart.clicked.connect(lambda: cu._select_fstart(self))
        freq = QtGui.QLineEdit(str(self.plot_state.fstart/constants.MHZ))
        def freq_change():
            cu._select_fstart(self)
            self.update_freq()
            self.update_freq_edit()
            
        freq.returnPressed.connect(lambda: freq_change())
        self._fstart_edit = freq
        self.control_widgets.append(self._fstart)
        self.control_widgets.append(self._fstart_edit)
        return fstart, freq
        
    def _fstop_controls(self):
        fstop = QtGui.QPushButton('Stop')
        fstop.setToolTip("[4]Tune the stop frequency") 
        self._fstop = fstop
        fstop.clicked.connect(lambda: cu._select_fstop(self))
        freq = QtGui.QLineEdit(str(self.plot_state.fstop/constants.MHZ))
        def freq_change():
            cu._select_fstop(self)   
            self.update_freq()
            self.update_freq_edit()            
        freq.returnPressed.connect(lambda: freq_change())
        self._fstop_edit = freq
        self.control_widgets.append(self._fstop)
        self.control_widgets.append(self._fstop_edit)
        return fstop, freq
           
    def _rbw_controls(self):
        rbw = QtGui.QComboBox(self)
        rbw.setToolTip("Change the RBW of the FFT plot")
        self._points_values = constants.RBW_VALUES
        self._rbw_box = rbw
        rbw.addItems([str(p) + ' KHz' for p in self._points_values])
        def new_rbw():
            self.plot_state.update_freq_set(rbw = self._points_values[rbw.currentIndex()])
        rbw.setCurrentIndex(0)
        rbw.currentIndexChanged.connect(new_rbw)
        self.control_widgets.append(self._rbw_box)
        return rbw

    def update_freq(self, delta = None):
            
        if delta == None:
            delta = 0                
        if self.plot_state.freq_sel == 'CENT':
            try:
                f = (float(self._freq_edit.text()) + delta) * constants.MHZ
            except ValueError:
                return
            if f > constants.MAX_FREQ or f < constants.MIN_FREQ:
                return
            self.plot_state.update_freq_set(fcenter = f)

        elif self.plot_state.freq_sel == 'FSTART':
            try:
                f = (float(self._fstart_edit.text()) + delta) * constants.MHZ 
            except ValueError:
                return
            if f > (constants.MAX_FREQ) or f < (constants.MIN_FREQ) or f > self.plot_state.fstop:
                return
            self.plot_state.update_freq_set(fstart = f)
            
        elif self.plot_state.freq_sel == 'FSTOP': 
            try:
                f = (float(self._fstop_edit.text()) + delta) * constants.MHZ 
            except ValueError:
                return
            if f > constants.MAX_FREQ or f < constants.MIN_FREQ or f < self.plot_state.fstart:
                return
            self.plot_state.update_freq_set(fstop = f)
        
        elif self.plot_state.freq_sel == 'BW':
            try:
                f = (float(self._bw_edit.text()) + delta) * constants.MHZ
            except ValueError:
                return
            if f < 0:
                return
            self.plot_state.update_freq_set(bw = f)
    
    def update_freq_edit(self):
        self._fstop_edit.setText("%0.1f" % (self.plot_state.fstop/ 1e6))
        self._fstart_edit.setText("%0.1f" % (self.plot_state.fstart/ 1e6))
        self._freq_edit.setText("%0.1f" % (self.plot_state.center_freq / 1e6))
        self._bw_edit.setText("%0.1f" % (self.plot_state.bandwidth / 1e6))
    
    def _plot_controls(self):

        plot_group = QtGui.QGroupBox("Plot Control")
        self._plot_group = plot_group
        
        plot_controls_layout = QtGui.QVBoxLayout()
        
        first_row = QtGui.QHBoxLayout()
        first_row.addWidget(self._marker_control())
        first_row.addWidget(self._delta_control())
        
        second_row = QtGui.QHBoxLayout()
        second_row.addWidget(self._peak_control())
        second_row.addWidget(self._mhold_control())
        
        third_row = QtGui.QHBoxLayout()
        third_row.addWidget(self._pause_control())
        third_row.addWidget(self._center_control())
        
        plot_controls_layout.addLayout(first_row)
        plot_controls_layout.addLayout(second_row)
        plot_controls_layout.addLayout(third_row)
        
        plot_group.setLayout(plot_controls_layout)
        
        return plot_group
    def _marker_control(self):
        marker = QtGui.QCheckBox('Marker 1')
        marker.setToolTip("[M]\nTurn Marker 1 on/off") 
        marker.clicked.connect(lambda: cu._marker_control(self))
        self._marker = marker
        self.control_widgets.append(self._marker)
        return marker
        
    def _delta_control(self):
        delta = QtGui.QCheckBox('Marker 2')
        delta.setToolTip("[K]\nTurn Marker 2 on/off") 
        delta.clicked.connect(lambda: cu._delta_control(self))
        self._delta = delta
        self.control_widgets.append(self._delta)
        return delta
    
    def _peak_control(self):
        peak = QtGui.QPushButton('Peak')
        peak.setToolTip("[P]\nFind peak of the selected spectrum") 
        peak.clicked.connect(lambda: cu._find_peak(self))
        self._peak = peak
        self.control_widgets.append(self._peak)
        return peak
        
    def _mhold_control(self):
        mhold = QtGui.QPushButton('Max Hold')
        mhold.setToolTip("[H]\nTurn the Max Hold on/off") 
        mhold.clicked.connect(lambda: cu._mhold_control(self))
        self._mhold = mhold
        self.control_widgets.append(self._mhold)
        return mhold
        
    def _center_control(self):
        center = QtGui.QPushButton('Recenter')
        center.setToolTip("[C]\nCenter the Plot View around the available spectrum") 
        center.clicked.connect(lambda: cu._center_plot_view(self))
        self._center_bt = center
        self.control_widgets.append(self._center_bt)
        return center
        
    def _pause_control(self):
        pause = QtGui.QPushButton('Pause')
        pause.setToolTip("[Space Bar]\n pause the plot window") 
        pause.clicked.connect(lambda: cu._enable_plot(self))
        self._pause = pause
        self.control_widgets.append(self._pause)
        return pause
    def _marker_labels(self):
        marker_label = QtGui.QLabel('')
        marker_label.setStyleSheet('color: %s;' % constants.TEAL)
        marker_label.setMinimumHeight(25)
        self._marker_lab = marker_label
        
        delta_label = QtGui.QLabel('')
        delta_label.setStyleSheet('color: %s;' % constants.TEAL)
        delta_label.setMinimumHeight(25)
        self._delta_lab = delta_label
        
        diff_label = QtGui.QLabel('')
        diff_label.setStyleSheet('color: %s;' % constants.TEAL)
        diff_label.setMinimumHeight(25)
        self._diff_lab = diff_label
        return marker_label,delta_label, diff_label
        
    def update_plot(self):
       
        self.plot_state.update_freq_range(self.plot_state.fstart,
                                              self.plot_state.fstop , 
                                              len(self.pow_data))

        self.update_fft()
        self.update_marker()
        self.update_delta()
        self.update_diff()
        
    def update_fft(self):

        if self.plot_state.mhold:
            if (self.plot_state.mhold_fft == None or len(self.plot_state.mhold_fft) != len(self.pow_data)):
                self.plot_state.mhold_fft = self.pow_data
            
            self.plot_state.mhold_fft = np.maximum(self.plot_state.mhold_fft,self.pow_data)
            self._plot.fft_curve.setData(x = self.plot_state.freq_range,
                                            y = self.plot_state.mhold_fft, 
                                            pen = constants.ORANGE_NUM)
        else:
            self._plot.fft_curve.setData(x = self.plot_state.freq_range, 
                                            y = self.pow_data, 
                                            pen = constants.TEAL_NUM)

    def update_trig(self):
            freq_region = self._plot.freqtrig_lines.getRegion()
            self.plot_state.trig_set = TriggerSettings(constants.LEVELED_TRIGGER_TYPE, 
                                                    min(freq_region), 
                                                    max(freq_region),
                                                    self._plot.amptrig_line.value())
            if self.plot_state.trig_set:
                self.dut.trigger(self.plot_state.trig_set)
    def update_marker(self):
        if self.plot_state.marker:
            if self.plot_state.mhold:
                pow_ = self.plot_state.mhold_fft
                self._marker_lab.setStyleSheet('color: %s;' % constants.ORANGE)
            else:
                pow_ = self.pow_data
                self._marker_lab.setStyleSheet('color: %s;' % constants.TEAL)
            if self.plot_state.marker_ind  == None:
                self.plot_state.marker_ind  = len(pow_) / 2 

            elif self.plot_state.marker_ind  < 0:
                self.plot_state.marker_ind  = 0
                
            elif self.plot_state.marker_ind  >= len(pow_):
                self.plot_state.marker_ind  = len(pow_) - 1
           
            marker_freq = [self.plot_state.freq_range[self.plot_state.marker_ind ]]
            markerpause_ffter = [pow_[self.plot_state.marker_ind]]
            marker_text = 'Frequency: %0.2f MHz \n Power %0.2f dBm' % (marker_freq[0]/1e6, markerpause_ffter[0])
            self._marker_lab.setText(marker_text)
            
            self._plot.marker_point.clear()
            self._plot.marker_point.addPoints(x = marker_freq, 
                                                    y = markerpause_ffter, 
                                                    symbol = '+', 
                                                    size = 20, pen = 'w', 
                                                    brush = 'w')

    def update_delta(self):
        if self.plot_state.delta:
            if self.plot_state.mhold:
                pow_ = self.plot_state.mhold_fft
                self._delta_lab.setStyleSheet('color: %s;' % constants.ORANGE)
            else:
                pow_ = self.pow_data
                self._delta_lab.setStyleSheet('color: %s;' % constants.TEAL)           
            
            if self.plot_state.delta_ind == None:
                self.plot_state.delta_ind = (len(pow_) / 2)
            elif self.plot_state.delta_ind < 0:
                self.plot_state.delta_ind = 0
                
            elif self.plot_state.delta_ind >= len(pow_):
                self.plot_state.delta_ind = len(pow_) - 1
            
            delta_freq = [self.plot_state.freq_range[self.plot_state.delta_ind]]
            delta_power = [pow_[self.plot_state.delta_ind]]
            delta_text = 'Frequency: %0.1f MHz \n Power %0.2f dBm' % (delta_freq[0]/1e6, delta_power[0])
            self._delta_lab.setText(delta_text)
           
            self._plot.delta_point.clear()
            self._plot.delta_point.addPoints(x =delta_freq, 
                                                    y = delta_power, 
                                                    symbol = '+', 
                                                    size = 20, pen = 'w', 
                                                    brush = 'w')

    def update_diff(self):
        if self.plot_state.mhold:
            pow_ = self.plot_state.mhold_fft
            self._diff_lab.setStyleSheet('color: %s;' % constants.ORANGE)
        else:
            pow_ = self.pow_data
            self._diff_lab.setStyleSheet('color: %s;' % constants.TEAL)  
            
        if self.plot_state.marker and self.plot_state.delta:
            freq_diff = np.abs((self.plot_state.freq_range[self.plot_state.delta_ind]/1e6) - (self.plot_state.freq_range[self.plot_state.marker_ind ]/1e6))
            power_diff = np.abs((pow_[self.plot_state.delta_ind]) - (pow_[self.plot_state.marker_ind ]))
            delta_text = 'Delta : %0.1f MHz \nDelta %0.2f dBm' % (freq_diff, power_diff )
            self._diff_lab.setText(delta_text)
        else:
            self._diff_lab.setText('')

    def enable_controls(self):
        for item in self.control_widgets:
            item.setEnabled(True)
        
    def disable_controls(self):
        for item in self.control_widgets:
            item.setEnabled(False)

        



import pyqtgraph as pg
import numpy as np
import constants

class plot(object):
    """
    Class to hold plot widget, as well as all the plot items (curves, marker_arrows,etc)
    """
    
    def __init__(self, layout):
    
        self.window = pg.PlotWidget(name='pyrf_plot')
        self.view_box = self.window.plotItem.getViewBox()
        # initialize the x-axis of the plot
        self.window.setLabel('bottom', text= 'Frequency', units = 'Hz', unitPrefix=None)

        # initialize the y-axis of the plot
        self.window.setYRange(constants.PLOT_YMIN, constants.PLOT_YMAX)
        self.window.setLabel('left', text = 'Power', units = 'dBm')
        
        # initialize fft curve
        self.fft_curve = self.window.plot(pen = constants.TEAL_NUM)
        self.marker_point = pg.ScatterPlotItem()
  
       # initialize marker
        self.marker_point = pg.ScatterPlotItem()

        # initialize delta
        self.delta_point = pg.ScatterPlotItem()
        
        # initialize trigger lines
        self.amptrig_line = pg.InfiniteLine(pos = -100, angle = 0, movable = True)
        self.freqtrig_lines = pg.LinearRegionItem()
        
        # update trigger settings when ever a line is changed
        self.freqtrig_lines.sigRegionChangeFinished.connect(layout.update_trig)
        self.amptrig_line.sigPositionChangeFinished.connect(layout.update_trig)
        self.grid(True)
    def add_marker(self):
        self.window.addItem(self.marker_point) 
        
    def remove_marker(self):
        self.window.removeItem(self.marker_point)
    
    def add_delta(self):
        self.window.addItem(self.delta_point) 
    
    def remove_delta(self):
        self.window.removeItem(self.delta_point)
        
    def add_trigger(self,fstart, fstop):
        self.freqtrig_lines.setRegion([fstart,fstop])
        self.window.addItem(self.amptrig_line)
        self.window.addItem(self.freqtrig_lines)
                
    def remove_trigger(self):
        self.window.removeItem(self.amptrig_line)
        self.window.removeItem(self.freqtrig_lines)
        
    def center_view(self,f,bw):
        self.window.setXRange(f - (bw/2),f + (bw / 2))
        self.window.setYRange(constants.PLOT_YMIN, constants.PLOT_YMAX)
        
    def grid(self,state):
        self.window.showGrid(state,state)


from collections import namedtuple

# Radio Front End modes

DC = 'direct_conversion'
SH = 'super_heterodyne'
HR = 'high_resolution'
DD = 'direct_digitization'
IQIN = 'iq_input'


RFEMode = namedtuple('RFEMode', """
    min_tunable
    max_tunable
    vrt_format
    usable_bw
    """, "Radio Front-End Mode properties")



import struct
import array
import sys

VRTCONTEXT = 4
VRTCUSTOMCONTEXT = 5
VRTDATA = 1

VRTRECEIVER = 0x90000001
VRTDIGITIZER = 0x90000002
VRTCUSTOM = 0x90000004
VRT_IFDATA_I14Q14 = 0x90000003
VRT_IFDATA_I14 = 0x90000005
VRT_IFDATA_I24 = 0x90000006
VRT_IFDATA_PSD8 = 0x90000007

CTX_REFERENCEPOINT = (1 << 30)
CTX_RFFREQ = (1 << 27)
CTX_GAIN = (1 << 23)
CTX_TEMPERATURE = (1 << 18)
CTX_BANDWIDTH = (1 << 29)
CTX_RFOFFSET = (1 << 26)
CTX_REFERENCELEVEL = (1 << 24)
CTX_SWEEPID = (1 << 0)
CTX_STREAMID = (1 << 1)

# values captured in a given frequency range
I_ONLY = 'i_only'
IQ = 'iq'

class InvalidDataReceived(Exception):
    pass


def vrt_packet_reader(raw_read):
    """
    Read a VRT packet, parse it and return an object with its data.

    Implemented as a generator that yields the result of the passed
    raw_read function and accepts the value sent as its data.
    """
    tmpstr = yield raw_read(4)
    (word,) = struct.unpack(">I", tmpstr)
    packet_type = (word >> 28) & 0x0f
    count = (word >> 16) & 0x0f
    size = (word >> 0) & 0xffff

    if packet_type in (VRTCONTEXT, VRTCUSTOMCONTEXT):
        packet_size = (size - 1) * 4
        context_data = yield raw_read(packet_size)
        yield ContextPacket(packet_type, count, size, context_data)

    elif packet_type == VRTDATA:
        data_header = yield raw_read(16)
        stream_id, tsi, tsf = struct.unpack(">IIQ", data_header)
        payload_size = (size - 5 - 1) * 4
        payload = yield raw_read(payload_size)
        trailer = yield raw_read(4)
        trailer = struct.unpack(">I", trailer)[0]
        yield DataPacket(count, size, stream_id, tsi, tsf, payload, trailer)

    else:
        raise InvalidDataReceived("unknown packet type: %s" % packet_type)



class ContextPacket(object):
    """
    A Context Packet received from :meth:`pyrf.devices.thinkrf.WSA.read`

    .. attribute:: fields

       a dict containing field names and values from the packet
    """

    def __init__(self, packet_type, count, size, tmpstr):
        self.ptype = packet_type
        self.count = count
        self.size = size
        (self.stream_id, self.tsi, self.tsf,indicatorsField,
            ) = struct.unpack(">IIQI", tmpstr[0:20])
        self.fields = {}

        # now read all the indicators
        if self.stream_id == VRTRECEIVER:
            self._parseReceiverContext(indicatorsField, tmpstr[20:])
        elif self.stream_id == VRTDIGITIZER:
            self._parseDigitizerContext(indicatorsField, tmpstr[20:])
        elif self.stream_id == VRTCUSTOM:
            self._parseCustomContext(indicatorsField, tmpstr[20:])


    def _parseReceiverContext(self, indicators, data):
        i = 0

        if indicators & CTX_REFERENCEPOINT:
            value = struct.unpack(">I", data[i:i+4])
            value = "0x%08x" % value
            self.fields['refpoint'] = value
            i += 4

        elif indicators & CTX_RFFREQ:
            (value,) = struct.unpack(">Q", data[i:i+8])
            value /= 2.0 ** 20
            self.fields['rffreq'] = value
            i += 8

        elif indicators & CTX_GAIN:
            (g1,g2) = struct.unpack(">hh", data[i:i+4])
            g1 /= 2.0 ** 7
            g2 /= 2.0 ** 7
            self.fields['gain'] = (g1, g2)
            i += 4

        elif indicators & CTX_TEMPERATURE:
            (value,) = struct.unpack(">I", data[i:i+4])
            value = value
            self.fields['temperature'] = value
            i += 4

        else:
            self.fields['unknown'] = (indicators, data)

    def _parseDigitizerContext(self, indicators, data):
        i = 0

        if indicators & CTX_BANDWIDTH:
            (value,) = struct.unpack(">Q", data[i:i+8])
            value /= 2.0 ** 20
            self.fields['bandwidth'] = value
            i += 8

        elif indicators & CTX_RFOFFSET:
            (value,) = struct.unpack(">q", data[i:i+8])
            value /= 2.0 ** 20
            self.fields['rfoffset'] = value
            i += 8

        elif indicators & CTX_REFERENCELEVEL:
            (value,) = struct.unpack(">h", data[i+2:i+4])
            value /= 2.0 ** 7
            self.fields['reflevel'] = value
            i += 4

        else:
            self.fields['unknown'] = (indicators, data)


    def _parseCustomContext(self, indicators, data):
        i = 0

        if indicators & CTX_SWEEPID:
            (value,) = struct.unpack(">I", data[i:i+4])
            self.fields['sweepid'] = value
            value = "0x%08x" % value
            self.fields['startid'] = value # backwards compat
            i += 4

        elif indicators & CTX_STREAMID:
            (value,) = struct.unpack(">I", data[i:i+4])
            self.fields['streamid'] = value
            i += 4

        else:
            self.fields['unknown'] = (indicators, data)


    def is_data_packet(self):
        """
        :returns: False
        """
        return False


    def is_context_packet(self, ptype=None):
        """
        :param ptype: "Receiver", "Digitizer" or None for any packet type

        :returns: True if this packet matches the type passed
        """
        if ptype is None:
            return True

        elif ptype == "Receiver":
            return self.ptype == VRTRECEIVER

        elif ptype == "Digitizer":
            return self.ptype == VRTDIGITIZER

        else:
            return False


    def __str__(self):
        return ("Context #%02d [%d.%012d, 0x%08x " % (
            self.count, self.tsi, self.tsf, self.stream_id)
            ) + str(self.fields) + "]"


class IQData(object):
    """
    Data Packet values as a lazy collection of (I, Q) tuples
    read from *binary_data*.

    This object behaves as an immutable python sequence, e.g.
    you may do any of the following:

    .. code-block:: python

       points = len(iq_data)

       i_and_q = iq_data[5]

       for i, q in iq_data:
           print i, q
    """
    def __init__(self, binary_data):
        self._strdata = binary_data
        self._data = None

    def _update_data(self):
        self._data = array.array('h')
        self._data.fromstring(self._strdata)
        if sys.byteorder == 'little':
            self._data.byteswap()

    def __len__(self):
        return len(self._strdata) / 4

    def __getitem__(self, n):
        if not self._data:
            self._update_data()
        return tuple(self._data[n * 2:n * 2 + 2])

    def __iter__(self):
        if not self._data:
            self._update_data()
        for i in range(0, len(self._data), 2):
            yield tuple(self._data[i:i + 2])

    def __reversed__(self):
        if not self._data:
            self._update_data()
        for i in reversed(range(0, len(self._data), 2)):
            yield tuple(self._data[i:i + 2])

    def numpy_array(self):
        """
        Return a numpy array of I, Q values for this data similar to:

        .. code-block:: python

           array([[ -44,    8],
                  [ -40,   60],
                  [ -12,   92],
                  ...,
                  [-132,   -8],
                  [-124,   56],
                  [ -44,   80]], dtype=int16)
        """
        import numpy
        a = numpy.frombuffer(self._strdata, dtype=numpy.int16)
        a = a.newbyteorder('>')
        a.shape = (-1, 2)
        return a


class DataArray(object):
    """
    Data Packet values as a lazy array read from *binary_data*.

    :param bytes_per_sample: 1 for PSD8 data, 2 for I14 data or
                             4 for I24 data
    """
    def __init__(self, binary_data, bytes_per_sample):
        self._strdata = binary_data
        self._bytes_per_sample = bytes_per_sample
        self._data = None

    def _update_data(self):
        self._data = array.array({
            1: 'b',
            2: 'h',
            4: 'l' if array.array('l').itemsize == 4 else 'i',
            }[self._bytes_per_sample])
        self._data.fromstring(self._strdata)
        if self._bytes_per_sample > 1 and sys.byteorder == 'little':
            self._data.byteswap()

    def __len__(self):
        return len(self._strdata) / self._bytes_per_sample

    def __getitem__(self, n):
        if not self._data:
            self._update_data()
        return self._data[n]

    def __iter__(self):
        if not self._data:
            self._update_data()
        return iter(self._data)

    def __reversed__(self):
        if not self._data:
            self._update_data()
        return reversed(self._data)

    def numpy_array(self):
        """
        return a numpy array for this data
        """
        import numpy
        a = numpy.frombuffer(self._strdata, dtype={
            1: numpy.int8,
            2: numpy.int16,
            4: numpy.int32,}[self._bytes_per_sample])
        if self._bytes_per_sample > 1:
            a = a.newbyteorder('>')
        return a


class DataPacket(object):
    """
    A Data Packet received from :meth:`pyrf.devices.thinkrf.WSA.read`

    .. attribute:: data

       a :class:`pyrf.vrt.IQData` object containing the packet data
    """

    def __init__(self, count, size, stream_id, tsi, tsf, payload, trailer):
        self.ptype = 1
        self.count = count
        self.size = size
        self.stream_id = stream_id
        self.tsi = tsi
        self.tsf = tsf

        # interpret data
        if self.stream_id == VRT_IFDATA_I14:
            self.data = DataArray(payload, 2)
        elif self.stream_id == VRT_IFDATA_PSD8:
            self.data = DataArray(payload, 1)
        elif self.stream_id == VRT_IFDATA_I24:
            self.data = DataArray(payload, 4)
        else:
            self.data = IQData(payload)

        self.valid_data = bool((trailer >> 18) & 1)
        self.reference_lock = bool((trailer >> 17) & 1)
        self.over_range = bool((trailer >> 13) & 1)
        self.sample_loss = bool((trailer >> 12) & 1)


    def is_data_packet(self):
        """
        :returns: True
        """
        return True


    def is_context_packet(self, ptype=None):
        """
        :returns: False
        """
        return False


    def __str__(self):
        return ("Data #%02d [%d.%012d, %d samples]" % (self.count, self.tsi, self.tsf, self.size - 6))



#!/usr/bin/env python

import sys
from pyrf.devices.thinkrf import WSA
import time

SAMPLES = 2**20

# connect to wsa
dut = WSA()
dut.connect(sys.argv[1])

# setup test conditions
dut.reset()
dut.request_read_perm()
dut.ifgain(0)
dut.freq(2450e6)
dut.gain('low')
dut.fshift(0)
dut.decimation(0)

for spp in [min(2**i, 2**16-16) for i in range(9, 17)]:
    dut.spp(spp)

    runs = max(SAMPLES/spp, 1)
    dut.stream_start(spp)
    start = None
    while True:
        pkt = dut.read()
        if pkt.is_context_packet() and pkt.fields.get('streamid') == spp:
            break
    start = time.time()
    for i in range(runs):
        while True:
            pkt = dut.read()
            if pkt.is_data_packet():
                break
    stop = time.time()
    dut.stream_stop()
    dut.flush()
    print 'spp %d: %f samples/second' % (spp, spp*runs/(stop-start))



from pyrf.config import TriggerSettings
import util
import pyqtgraph as pg
import gui_config as gui_state
import constants
from pyrf.util import read_data_and_context
def _center_plot_view(layout):
    """
    move the view to the center of the current FFT displayed
    """
    layout._plot.center_view(layout.plot_state.center_freq, layout.plot_state.bandwidth)
    
def _select_center_freq(layout):
    """
    select the center freq for arrow control
    """
    layout.plot_state.freq_sel = 'CENT'
    util.select_center(layout)
    
def _select_bw(layout):
    """
    select the bw for arrow control
    """
    layout.plot_state.freq_sel = 'BW'
    util.select_bw(layout)

def _select_fstart(layout):
    """
    select the fstart for arrow control
    """
    layout.plot_state.freq_sel = 'FSTART'
    util.select_fstart(layout)
    
def _select_fstop(layout):
    """
    select the fstop for arrow control
    """
    layout.plot_state.freq_sel = 'FSTOP'
    util.select_fstop(layout)
    
def _up_arrow_key(layout):
    """
    increase the step size of the +/- buttons
    """
    step = layout._fstep_box.currentIndex() + 1
    max_step = layout._fstep_box.count()
    if step > max_step - 1:
        step = max_step -1
    elif step < 0:
        step = 0
        layout._fstep_box.setCurrentIndex(step)
    layout._fstep_box.setCurrentIndex(step)

def _down_arrow_key(layout):
    """
    decrease the step size of the +/- buttons
    """
    step = layout._fstep_box.currentIndex() - 1
    max_step = layout._fstep_box.count()
    if step > max_step - 1:
        step = max_step -1
    elif step < 0:
        step = 0
        layout._fstep_box.setCurrentIndex(step)
    layout._fstep_box.setCurrentIndex(step)
       
def _right_arrow_key(layout):
    """
    handle arrow key right action
    """

    if layout.plot_state.enable_plot:
        layout._freq_plus.click()
        layout.plot_state.mhold_fft = None

def _left_arrow_key(layout):
    """
    handle left arrow key action
    """
    if layout.plot_state.enable_plot:
        layout._freq_minus.click()
        layout.plot_state.mhold_fft = None

def _mhold_control(layout):
    """
    disable/enable max hold curve in the plot
    """
    if layout.plot_state.enable_plot:
        layout.plot_state.mhold = not(layout.plot_state.mhold)
            
        if layout.plot_state.mhold:
            util.change_item_color(layout._mhold,  constants.ORANGE, constants.WHITE)           
        else:  
            util.change_item_color(layout._mhold,  constants.NORMAL_COLOR, constants.BLACK)
            layout.plot_state.mhold_fft = None
        
def _marker_control(layout):
    """
    disable/enable marker
    """
    # if marker is on and selected, turn off
    if layout.plot_state.marker_sel:
        layout.plot_state.disable_marker(layout)

            
    # if marker is on and not selected, select
    elif not layout.plot_state.marker_sel and layout.plot_state.marker: 
        layout.plot_state.enable_marker(layout)

    # if marker is off, turn on and select
    elif not layout.plot_state.marker:
        layout.plot_state.enable_marker(layout)

def _delta_control(layout):
    """
    disable/enable delta (marker 2)
    """

    # if delta is on and selected, turn off
    if layout.plot_state.delta_sel:
        layout.plot_state.disable_delta(layout)
    
    # if delta is on and not selected, select
    elif not layout.plot_state.delta_sel and layout.plot_state.delta: 
        layout.plot_state.enable_delta(layout)

    # if delta is off, turn on and select
    elif not layout.plot_state.delta:
        layout.plot_state.enable_delta(layout)   

def _find_peak(layout):
    """
    move the selected marker to the maximum point of the spectrum
    """
    if not layout.plot_state.marker and not layout.plot_state.delta:
        _marker_control(layout)

    if layout.plot_state.mhold:
       peak = util.find_max_index(layout.plot_state.mhold_fft) 
    else:
        peak = util.find_max_index(layout.pow_data)
    
    if layout.plot_state.marker_sel:
        layout.update_marker()
        layout.plot_state.marker_ind = peak
    elif layout.plot_state.delta_sel:
        layout.update_delta()
        layout.plot_state.delta_ind = peak
    layout.update_diff()
def _enable_plot(layout):
    """
    pause/unpause the plot
    """
    layout.plot_state.enable_plot = not(layout.plot_state.enable_plot)
    if not layout.plot_state.enable_plot:
        util.change_item_color(layout._pause,  constants.ORANGE, constants.WHITE)
        
    else:
        util.change_item_color(layout._pause,  constants.NORMAL_COLOR, constants.BLACK)
        layout.read_sweep()

def _trigger_control(layout):
    """
    disable/enable triggers in the layout plot
    """

    if layout.plot_state.trig:
        layout.plot_state.disable_trig(layout)
           
    else:
        layout.plot_state.enable_trig(layout)
        _select_center_freq(layout)
        
hotkey_dict = {'1': _select_fstart,
                '2': _select_center_freq,
                '3': _select_bw,
                '4': _select_fstop,
                'UP KEY': _up_arrow_key, 
                'DOWN KEY': _down_arrow_key,
                'RIGHT KEY': _right_arrow_key,
                'LEFT KEY': _left_arrow_key,
                'C': _center_plot_view,
                'K': _delta_control,
                'H': _mhold_control,
                'M': _marker_control,
                'P': _find_peak,
                'SPACE': _enable_plot,
                'T': _trigger_control
                } 
                
arrow_dict = {'32': 'SPACE', 
                '16777235': 'UP KEY', 
                '16777237': 'DOWN KEY',
                '16777234': 'LEFT KEY', 
                '16777236': 'RIGHT KEY'}





#!/usr/bin/env python

from pyrf.devices.thinkrf import WSA
from pyrf.sweep_device import SweepDevice

import sys
import time
import math

from matplotlib.pyplot import plot, figure, axis, xlabel, ylabel, show
import numpy as np

# connect to wsa
dut = WSA()
dut.connect(sys.argv[1])
sd = SweepDevice(dut)

fstart, fstop, bins = sd.capture_power_spectrum(0e9, 10e9, 5e6,
    {'gain': 'high', 'antenna': 1})


# setup my graph
fig = figure(1)
xvalues = np.linspace(fstart, fstop, len(bins))

xlabel("Frequency")
ylabel("Amplitude")

# plot something
plot(xvalues, bins, color='blue')

# show graph
show()



# WSA Constants
MIN_FREQ = 45e6
MAX_FREQ = 10e9
MAX_BW = MAX_FREQ - MIN_FREQ
MIN_BW = 1e5
DEVICE_FULL_SPAN = 125e6
STARTUP_POINTS = 1024
LEVELED_TRIGGER_TYPE = 'LEVEL'
NONE_TRIGGER_TYPE = 'NONE'
MHZ = 1e6
INIT_CENTER_FREQ = 2450 * MHZ
INIT_BANDWIDTH = 500 * MHZ

RBW_VALUES = [976.562, 488.281, 244.141, 122.070, 61.035, 30.518, 15.259, 7.629, 3.815]
INIT_BIN_SIZE = DEVICE_FULL_SPAN / (RBW_VALUES[2] * 1e3)
# Plot Constants
PLOT_YMIN = -160
PLOT_YMAX = 20
LNEG_NUM = -5000

# Colors
NORMAL_COLOR = 'NONE'
ORANGE =  'rgb(255,84,0)'
ORANGE_NUM = (255,84,0)
TEAL = 'rgb(0,255,236)'
TEAL_NUM = (0,255,236)
WHITE = 'rgb(255,255,255)'
BLACK = 'rgb(0,0,0)'

PRESSED_BUTTON = 'groove'


from pyrf.numpy_util import compute_fft
from pyrf.config import TriggerSettings

import math


class CaptureDeviceError(Exception):
    pass


class CaptureDevice(object):
    """
    Virtual device that returns power levels generated from a single data packet
    :param real_device: device that will will be used for capturing data,
                        typically a :class:`pyrf.thinkrf.WSA` instance.
    :param callback: callback to use for async operation (not used if
                     real_device is using a :class:`PlainSocketConnector`)

    """    
    def __init__(self, real_device, async_callback=None):
        
        self.real_device = real_device
        self.connector = self.real_device.connector
        if hasattr(self.connector, 'vrt_callback'):
            if not async_callback:
                raise CaptureDeviceError(
                    "async_callback required for async operation")
            # disable receiving data until we are expecting it
            self.connector.vrt_callback = None
        else:
            if async_callback:
                raise CaptureDeviceError(
                    "async_callback not applicable for sync operation")
        self.async_callback = async_callback


    def capture_power_spectrum(self, device_set, rbw, min_points=128):
        """
        Initiate a capture of power spectral density by
        applying leveled triggers and return the first data packet that
        satisfy the trigger.

        :param fstart: starting frequency in Hz
        :type fstart: float
        :param fstop: ending frequency in Hz
        :type fstop: float
        :param rbw: requested RBW in Hz (output RBW may be smaller than
                    requested)
        :type rbw: float
        :param triggers: a class containing trigger information
        :type class:`TriggerSettings`
        :param device_settings: antenna, gain and other device settings
        :type dict:
        """

        prop = self.real_device.properties

        # setup the WSA device
        self.fstart = device_set['freq'] - prop.USABLE_BW / 2
        self.fstop =  device_set['freq'] + prop.USABLE_BW / 2
        self.real_device.apply_device_settings(device_set)
        
        self.real_device.abort()
        self.real_device.flush()
        self.real_device.request_read_perm()
        self._vrt_context = {}

        points = prop.FULL_BW / rbw
        points = max(min_points, 2 ** math.ceil(math.log(points, 2)))

        if self.async_callback:

            self.connector.vrt_callback = self.read_data
            self.real_device.capture(points, 1)

            return

        self.real_device.capture(points, 1)
        result = None
        while result is None:
            result = self.read_data(self.real_device.read())
        return result

    def read_data(self, packet):
        if packet.is_context_packet():
            self._vrt_context.update(packet.fields)
            return

        pow_data = compute_fft(self.real_device, packet, self._vrt_context)
        prop = self.real_device.properties
        attenuated_edge = math.ceil((1.0 -
            float(prop.USABLE_BW) / prop.FULL_BW) / 2 * len(pow_data))
        pow_data = pow_data[attenuated_edge:-attenuated_edge]
        # FIXME: fstart and fstop not properly corrected for bins removed

        if self.async_callback:
            self.async_callback(self.fstart, self.fstop, pow_data)
            return
        return (self.fstart, self.fstop, pow_data)



import socket

from pyrf.connectors.base import sync_async, SCPI_PORT, VRT_PORT

import logging
logger = logging.getLogger(__name__)

class PlainSocketConnector(object):
    """
    This connector makes SCPI/VRT socket connections using plain sockets.
    """

    def connect(self, host):
        self._sock_scpi = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._sock_scpi.connect((host, SCPI_PORT))
        self._sock_scpi.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, True)
        self._sock_vrt = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._sock_vrt.connect((host, VRT_PORT))

    def disconnect(self):
        self._sock_scpi.shutdown(socket.SHUT_RDWR)
        self._sock_scpi.close()
        self._sock_vrt.shutdown(socket.SHUT_RDWR)
        self._sock_vrt.close()

    def scpiset(self, cmd):
        cmd = "%s\n" % cmd
        logger.debug('scpiset %r', cmd)
        self._sock_scpi.send(cmd)

    def scpiget(self, cmd):
        cmd = "%s\n" % cmd
        logger.debug('scpiset %r', cmd)
        self._sock_scpi.send(cmd)
        buf = self._sock_scpi.recv(1024)
        logger.debug('scpigot %r', buf)
        return buf

    def eof(self):
        # FIXME: lies
        return False

    def has_data(self):
        # FIXME: broken
        return self._vrt.has_data()

    def raw_read(self, num):
        return socketread(self._sock_vrt, num)

    def sync_async(self, gen):
        """
        Handler for the @sync_async decorator.  We convert the
        generator to a single return value for simple synchronous use.
        """
        val = None
        try:
            while True:
                val = gen.send(val)
        except StopIteration:
            return val


def socketread(socket, count, flags = None):
    """
    Retry socket read until count data received,
    like reading from a file.
    """
    if not flags:
        flags = 0
    data = socket.recv(count, flags)
    datalen = len(data)

    if datalen == 0:
        return False

    while datalen < count:
        data = data + socket.recv(count - datalen)
        datalen = len(data)

    return data



#!/usr/bin/python

from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor
import struct
import platform

from pyrf.devices.thinkrf import (DISCOVERY_UDP_PORT, DISCOVERY_QUERY,
    parse_discovery_response)

WAIT_TIME = 0.125

class DiscoverWSAs(DatagramProtocol):
    def startProtocol(self):
        import socket
        self.transport.socket.setsockopt(
            socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        if platform.system() == 'Windows':
            import _windows_networks
            destinations = _windows_networks.get_broadcast_addresses()
        else:
            destinations = ['<broadcast>']
        for d in destinations:
            self.transport.socket.sendto(DISCOVERY_QUERY,
                (d, DISCOVERY_UDP_PORT))

    def datagramReceived(self, data, (host, port)):
        model, serial, firmware = parse_discovery_response(data)
        print model, serial, firmware, 'at', host

reactor.listenUDP(0, DiscoverWSAs())
reactor.callLater(WAIT_TIME, reactor.stop)
reactor.run()



try:
    from twisted.internet.protocol import Factory, Protocol
    from twisted.internet import defer
    from twisted.internet.endpoints import TCP4ClientEndpoint
    try: # Twisted >= 13.0 for IPv6 support
        from twisted.internet.endpoints import HostnameEndpoint
    except ImportError:
        from twisted.internet.endpoints import TCP4ClientEndpoint
        HostnameEndpoint = TCP4ClientEndpoint
except ImportError:
    # to allow docstrings to be visible even when twisted
    # imports fail
    Factory = Protocol = StatefulProtocol = object

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from pyrf.connectors.base import sync_async, SCPI_PORT, VRT_PORT
from pyrf.vrt import vrt_packet_reader

import logging
logger = logging.getLogger(__name__)

class TwistedConnectorError(Exception):
    pass

class TwistedConnector(object):
    """
    A connector that makes SCPI/VRT connections asynchronously using
    Twisted.

    A callback may be assigned to vrt_callback that will be called
    with VRT packets as they arrive.  When .vrt_callback is None
    (the default) arriving packets will be ignored.
    """
    def __init__(self, reactor, vrt_callback=None):
        self._reactor = reactor
        self.vrt_callback = vrt_callback

    def connect(self, host):
        point = HostnameEndpoint(self._reactor, host, SCPI_PORT)
        d = point.connect(SCPIClientFactory())

        @d.addCallback
        def connect_vrt(scpi):
            self._scpi = scpi
            point = HostnameEndpoint(self._reactor, host, VRT_PORT)
            return point.connect(VRTClientFactory(self._vrt_callback))

        @d.addCallback
        def save_vrt(vrt):
            self._vrt = vrt

        return d

    def disconnect(self):
        pass # FIXME

    def scpiset(self, cmd):
        self._scpi.scpiset("%s\n" % cmd)

    def scpiget(self, cmd):
        return self._scpi.scpiget("%s\n" % cmd)

    def sync_async(self, gen):
        def advance(result):
            try:
                d = gen.send(result)
                d = defer.maybeDeferred(lambda: d)
            except StopIteration:
                return result
            d.addCallback(advance)
            return d

        return advance(None)

    def eof(self):
        return self._vrt.eof

    def raw_read(self, num_bytes):
        raise TwistedConnectorError('synchronous read() not supported.')

    def _vrt_callback(self, packet):
        if self.vrt_callback:
            self.vrt_callback(packet)


class VRTClient(Protocol):
    """
    A Twisted protocol for the VRT connection

    :param receive_callback: a function that will be passed a vrt
        DataPacket or ContextPacket when it is received
    """
    _buf = None
    eof = False

    def __init__(self, receive_callback):
        self._receive_callback = receive_callback

    def makeConnection(self, transport):
        Protocol.makeConnection(self, transport)
        self._buf = StringIO()
        self._buf_offset = 0
        self._resetReader()
        self._processData()

    def _resetReader(self):
        self._packet_reader = vrt_packet_reader(self._setBytesRequired)
        next(self._packet_reader) 

    def _setBytesRequired(self, x):
        self._bytes_required = x

    def _processData(self):
        """
        If we have received enough bytes process it as VRT data and
        call receive_callback if a complete packet was received.
        """
        while True:
            data = self._bufConsume(self._bytes_required)
            if not data:
                break
            response = self._packet_reader.send(data)
            if response:
                self._receive_callback(response)
                self._resetReader()

    def _bufAppend(self, data):
        if self._buf_offset:
            self._buf.seek(self._buf_offset)
            prefix = self._buf.read()
            self._buf.seek(0)
            self._buf.truncate()
            self._buf.write(prefix)
            self._buf_offset = 0
        self._buf.seek(0, 2)
        self._buf.write(data)

    def _bufConsume(self, num_bytes):
        "returns None if not enough bytes available"
        if self._bufLength() < num_bytes:
            return None
        self._buf.seek(self._buf_offset)
        self._buf_offset += num_bytes
        return self._buf.read(num_bytes)

    def _bufLength(self):
        self._buf.seek(0, 2)
        return self._buf.tell() - self._buf_offset

    def dataReceived(self, data):
        self._bufAppend(data)
        self._processData()

    def connectionLost(self, reason):
        self.eof = True

class VRTClientFactory(Factory):
    def __init__(self, receive_callback):
        self._receive_callback = receive_callback

    def startedConnecting(self, connector):
        pass

    def buildProtocol(self, addr):
        return VRTClient(self._receive_callback)

    def clientConnectionLost(self, connector, reason):
        pass

    def clientConnectionFailed(self, connector, reason):
        pass

class SCPIClient(Protocol):
    _pending = None

    def connectionMade(self):
        self.transport.setTcpNoDelay(True)

    def scpiset(self, cmd):
        if self._pending:
            # prevent reordering
            self._pending.append((cmd, None))
        else:
            logger.debug('scpiset %r', cmd)
            self.transport.write(cmd)

    def scpiget(self, cmd):
        d = defer.Deferred()
        if self._pending:
            # command pipelining not supported
            self._pending.append((cmd, d))
        else:
            self._pending = [('', d)]
            self.transport.write(cmd)
            logger.debug('scpiget %r', cmd)
        return d

    def dataReceived(self, data):
        cmd, d = self._pending.pop(0)
        logger.debug('scpigot %r', data)
        if d:
            d.callback(data)

        while self._pending:
            cmd, d = self._pending[0]
            logger.debug('scpi(%s) %r', 'get' if d else 'set', cmd)
            self.transport.write(cmd)
            if d:
                break
            self._pending.pop(0)


class SCPIClientFactory(Factory):
    def startedConnecting(self, connector):
        pass
    def buildProtocol(self, addr):
        return SCPIClient()
    def clientConnectionLost(self, connector, reason):
        pass
    def clientConnectionFailed(self, connector, reason):
        pass





def select_fstart(layout):
    layout._fstart.setStyleSheet("Background-color: green")
    layout._cfreq.setStyleSheet("Background-color: None")
    layout._fstop.setStyleSheet("Background-color: None")

def select_center(layout):
    layout._cfreq.setStyleSheet("Background-color: green")
    layout._fstart.setStyleSheet("Background-color: None")
    layout._fstop.setStyleSheet("Background-color: None")

def select_fstop(layout):
    layout._fstop.setStyleSheet("Background-color: Green")
    layout._fstart.setStyleSheet("Background-color: None")
    layout._cfreq.setStyleSheet("Background-color: None")

def change_item_color(item, textColor, backgroundColor):
    item.setStyleSheet("Background-color: %s; color: %s" % (textColor, backgroundColor)) 




import constants
import util
import numpy as np
from pyrf.config import TriggerSettings
class plot_state(object):
    """
    Class to hold all the GUI's plot states
    """

    def __init__(self):
        
        self.grid = False
        self.dev_set = {
            'gain' : 'vlow',
            'antenna' : 1,
            'ifgain' : 0,
            'attenuator': 1} 
        self.mhold = False
        self.mhold_fft = None
        
        
        
        self.trig = False
        self.trig_set = None
        self.marker = False
        self.marker_sel = False
        self.marker_ind = None
        
        self.delta = False
        self.delta_sel = False
        self.delta_ind = None
        self.peak = False
        
        self.freq_range = None        
        self.center_freq = constants.INIT_CENTER_FREQ
        self.bandwidth = constants.INIT_BANDWIDTH
        self.fstart = self.center_freq - self.bandwidth / 2
        self.fstop = self.center_freq + self.bandwidth / 2
        self.bin_size = constants.INIT_BIN_SIZE
        self.rbw = self.bandwidth / self.bin_size
        self.enable_plot = True
        self.freq_sel = 'CENT'
    
    def enable_marker(self, layout):
        self.marker = True
        self.marker_sel = True
        util.change_item_color(layout._marker,  constants.ORANGE, constants.WHITE)
        layout._plot.add_marker()
        layout._marker.setDown(True)
        layout.update_marker()
        if layout.plot_state.delta_sel:
            self.delta_sel = False
            util.change_item_color(layout._delta,  constants.ORANGE, constants.WHITE)
            layout._delta.setDown(False)
            

    def disable_marker(self, layout):
        
        self.marker = False
        self.marker_sel = False
        util.change_item_color(layout._marker, constants.NORMAL_COLOR, constants.BLACK)
        layout._marker.setDown(False)
        layout._plot.remove_marker()
        layout._marker_lab.setText('')
        layout._plot.center_view(layout.plot_state.center_freq, layout.plot_state.bandwidth)
        if self.delta:
            self.enable_delta(layout)

    def enable_delta(self, layout):
        self.delta = True
        self.delta_sel = True
        util.change_item_color(layout._delta, constants.ORANGE, constants.WHITE)
        layout._plot.add_delta()
        layout._delta.setDown(True)
        layout.update_delta()
        if self.marker:
            self.marker_sel = False             
            util.change_item_color(layout._marker, constants.ORANGE, constants.WHITE)
            layout._marker.setDown(False)
            
    def disable_delta(self, layout):
        self.delta = False
        self.delta_sel = False
        util.change_item_color(layout._delta, constants.NORMAL_COLOR ,constants.BLACK)
        layout._delta.setDown(False)
        layout._plot.remove_delta()
        layout._delta_lab.setText('')
        layout._diff_lab.setText('')
        layout._plot.center_view(layout.plot_state.center_freq, layout.plot_state.bandwidth)
        if self.marker:
            self.enable_marker(layout)
    
    def disable_trig(self, layout):
        self.trig = False
        util.change_item_color(layout._trigger, constants.NORMAL_COLOR, constants.BLACK)
        layout._plot.remove_trigger()
        self.trig_set = None
        util.enable_freq_cont(layout)
        
    def enable_trig(self, layout):
        self.trig = True
        util.change_item_color(layout._trigger, constants.ORANGE,constants.WHITE)

        self.trig_set = TriggerSettings(constants.LEVELED_TRIGGER_TYPE,
                                                self.center_freq + 10e6, 
                                                self.center_freq - 10e6,-100)
        
        layout.plot_state.freq_sel = 'BW'
        layout._bw_edit.setText('100.0')
        layout.update_freq()
        layout.update_freq_edit()
        util.disable_freq_cont(layout)
        layout._plot.add_trigger(self.trig_set.fstart, self.trig_set.fstop)
        
    def update_freq_range(self, start, stop, size):
        self.freq_range = np.linspace(start, stop, size)
        


    def update_freq_set(self,
                          fstart = None, 
                          fstop = None, 
                          fcenter = None, 
                          rbw = None, 
                          bw = None):
        
        if fcenter != None:
            self.fstart = fcenter - (self.bandwidth / 2)
            if self.fstart < constants.MIN_FREQ:
                self.fstart = constants.MIN_FREQ
            self.fstop = fcenter + (self.bandwidth / 2)
            if self.fstop > constants.MAX_FREQ:    
                self.fstop = constants.MAX_FREQ
            self.bandwidth = self.fstop - self.fstart
            self.center_freq = self.fstart + (self.bandwidth / 2)
            self.bin_size = int((self.bandwidth) / self.rbw)
            if self.bin_size < 1:
                self.bin_size = 1
        
        elif fstart != None:
            if fstart >= self.fstop:
                fstart = self.fstop - constants.MIN_BW
            self.fstart = fstart
            self.bandwidth = self.fstop - fstart
            self.center_freq = fstart + (self.bandwidth / 2)
            self.bin_size = int((self.bandwidth) / self.rbw)
            if self.bin_size < 1:
                self.bin_size = 1
                
        elif fstop != None:
            if fstop <= self.fstart:
                fstop = self.fstart + constants.MIN_BW
            self.fstop = fstop
            self.bandwidth = fstop - self.fstart
            self.center_freq = fstop - (self.bandwidth / 2)
            self.bin_size = int((self.bandwidth) / self.rbw)
            if self.bin_size < 1:
                self.bin_size = 1
                
        elif rbw != None:
            self.rbw = rbw * 1e3
            self.bin_size = int((self.bandwidth) / self.rbw)
            if self.bin_size < 1:
                self.bin_size = 1
        
        elif bw != None:
            if bw < constants.MIN_BW:
                bw = constants.MIN_BW
            self.fstart = (self.center_freq - (bw / 2))
            self.fstop = (self.center_freq + (bw / 2))
            if self.fstart < constants.MIN_FREQ:
                self.fstart = constants.MIN_FREQ
            if self.fstop > constants.MAX_FREQ:    
                self.fstop = constants.MAX_FREQ
            self.bandwidth = self.fstop - self.fstart
            self.center_freq = self.fstart + (self.bandwidth / 2)
            self.bin_size = int((self.bandwidth) / self.rbw)
            if self.bin_size < 1:
                self.bin_size = 1
                
    def reset_freq_bounds(self):
            self.start_freq = None
            self.stop_freq = None

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    



#!/usr/bin/env python

from pyrf.devices.thinkrf import WSA
from pyrf.connectors.twisted_async import TwistedConnector
from pyrf.sweep_device import SweepDevice

import sys
import time
import math

from matplotlib.pyplot import plot, figure, axis, xlabel, ylabel, show
import numpy as np

from twisted.internet import reactor, defer
import twisted.python.log

def plot_sweep(fstart, fstop, bins):
    # setup my graph
    fig = figure(1)
    xvalues = np.linspace(fstart, fstop, len(bins))

    xlabel("Frequency")
    ylabel("Amplitude")

    # plot something
    plot(xvalues, bins, color='blue')

    # show graph
    show()
    reactor.callLater(2 ** -4, reactor.stop)

def start_sweep(v):
    global sd
    sd = SweepDevice(dut, plot_sweep)
    sd.capture_power_spectrum(0e9, 10e9, 5e6,
        {'gain': 'high', 'antenna': 1})

# connect to wsa
dut = WSA(connector=TwistedConnector(reactor))
d = dut.connect(sys.argv[1])
d.addCallbacks(start_sweep, twisted.python.log.err)
reactor.run()

print 'context_bytes_received', sd.context_bytes_received
print 'data_bytes_received', sd.data_bytes_received
print 'data_bytes_processed', sd.data_bytes_processed
print 'martian_bytes_discarded', sd.martian_bytes_discarded
print 'past_end_bytes_discarded', sd.past_end_bytes_discarded
print 'fft_calculation_seconds', sd.fft_calculation_seconds
print 'bin_collection_seconds', sd.bin_collection_seconds




#!/usr/bin/python
import socket
import struct
import select
import platform

from pyrf.devices.thinkrf import (DISCOVERY_UDP_PORT, DISCOVERY_QUERY,
    parse_discovery_response)


WAIT_TIME = 0.125

cs = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
cs.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
cs.setblocking(0)

if platform.system() == 'Windows':
    import _windows_networks
    destinations = _windows_networks.get_broadcast_addresses()
else:
    destinations = ['<broadcast>']

for d in destinations:
    # send query command to WSA
    query_struct = DISCOVERY_QUERY
    cs.sendto(query_struct, (d, DISCOVERY_UDP_PORT))

while True:
    ready, _, _ = select.select([cs], [], [], WAIT_TIME)
    if not ready:
        break
    data, (host, port) = cs.recvfrom(1024)

    model, serial, firmware = parse_discovery_response(data)

    print model, serial, firmware, 'at', host




import  control_util 
import numpy as np
import constants
def frequency_text(hz):
    """
    return hz as readable text in Hz, kHz, MHz or GHz
    """
    if hz < 1e3:
        return "%.3f Hz" % hz
    elif hz < 1e6:
        return "%.3f kHz" % (hz / 1e3)
    elif hz < 1e9:
        return "%.3f MHz" % (hz / 1e6)
    return "%.3f GHz" % (hz / 1e9)
    
def hotkey_util(layout,event):
    """
    modify elements in the gui layout based on which key was pressed
    """
    if control_util.arrow_dict.has_key(str(event.key())):
        hotkey =  control_util.arrow_dict[str(event.key())]
    else:
        hotkey = str(event.text()).upper()
    if control_util.hotkey_dict.has_key(hotkey):
        control_util.hotkey_dict[hotkey](layout)
        
def find_max_index(array):
    """
    returns the maximum index of an array         
    """
    # keep track of max index
    index = 0
    
    array_size = len(array)
    
    max_value = 0
    for i in range(array_size):
        
        if i == 0:
            max_value = array[i]
            index = i
        elif array[i] > max_value:
            max_value = array[i]
            index = i
    return index

def find_nearest_index(value, array):
    """
    returns the index in the array of the nearest value      
    """
    idx = (np.abs(array-value)).argmin()
    return idx
    
def select_fstart(layout):
    layout._fstart.setStyleSheet('background-color: %s; color: white;' % constants.ORANGE)
    layout._cfreq.setStyleSheet("")
    layout._fstop.setStyleSheet("")
    layout._bw.setStyleSheet("")
    
def select_center(layout):
    layout._cfreq.setStyleSheet('background-color: %s; color: white;' % constants.ORANGE)
    layout._fstart.setStyleSheet("")
    layout._fstop.setStyleSheet("")
    layout._bw.setStyleSheet("")
    
def select_bw(layout):
    layout._bw.setStyleSheet('background-color: %s; color: white;' % constants.ORANGE)
    layout._fstart.setStyleSheet("")
    layout._cfreq.setStyleSheet("")
    layout._fstop.setStyleSheet("")

def select_fstop(layout):
    layout._fstop.setStyleSheet('background-color: %s; color: white;' % constants.ORANGE)
    layout._fstart.setStyleSheet("")
    layout._cfreq.setStyleSheet("")
    layout._bw.setStyleSheet("")
    
def change_item_color(item, textColor, backgroundColor):
    item.setStyleSheet("QPushButton{Background-color: %s; color: %s; } QToolButton{color: Black}" % (textColor, backgroundColor)) 

def enable_freq_cont(layout):
    layout._bw.setEnabled(True)
    layout._bw_edit.setEnabled(True)
    layout._fstart.setEnabled(True)
    layout._fstart_edit.setEnabled(True)
    layout._fstop.setEnabled(True)
    layout._fstop_edit.setEnabled(True)
    
def disable_freq_cont(layout):
    layout._bw.setEnabled(False)
    layout._bw_edit.setEnabled(False)
    layout._fstart.setEnabled(False)
    layout._fstart_edit.setEnabled(False)
    layout._fstop.setEnabled(False)
    layout._fstop_edit.setEnabled(False)


